/* This file converts the flat file from XSRAM of size 32K to the
 * corresponding .rgb file for a 512x256 pixel image. The default flatfile
 * name is "mazefile" or can be specified as a command line paramter.
 * Depending on the input file name, the corresponding .rgb (mazefile.rgb"
 * by default) is created. The .rgb file can be viewed using ImageMagick. On
 * Linux it accessed by the command "display" (display -size 512x256
 * <filename>.rgb, to be exact). ImageMagick is also available for NT/95
 * from their website
 * "http://www.wizards.dupont.com/cristy/ImageMagick.html". The convert code
 * is compilable on Linux (gcc) as well on NT (VC++ 5.0).
 *
 */

#include <stdio.h>
#include <string.h>

#define WALL 0x40
#define NO_WALL 0x01
#define NORTH	0x02
#define WEST	0x04
#define SOUTH	0x08
#define EAST	0x10
#define EXIT	0x20
#define CRUMB 	0xFF
#define NO_CRUMB 0x00

#define MAX_ARRAY_SIZE 128


#define FILENAME "mazefile"

char maze_array[2*MAX_ARRAY_SIZE][4*MAX_ARRAY_SIZE];

int main(int argc,char **argv){

    char name[30];
    char filename[30];
    char inputc, out1, out2, out3;
    int i,j,l,m;

    FILE *fp1, *fp2;

    if(argc == 2) strcpy(name,argv[1]);
    else strcpy(name,FILENAME);

    strcpy(filename,name);
    strcat(filename,".rgb");

    fp1 = fopen(name,"r");
    fp2 = fopen(filename,"w");
    
    for(i=0;i<2*MAX_ARRAY_SIZE;i++){
	for(j=0;j<MAX_ARRAY_SIZE;j++){
	    l = MAX_ARRAY_SIZE - 1 - j; m = i;
	    inputc = (char) fgetc(fp1);
	    maze_array[2*l][2*m] = inputc;
	    maze_array[2*l+1][2*m] = inputc;
	    maze_array[2*l][2*m+1] = inputc;
	    maze_array[2*l+1][2*m+1] = inputc;
	}
    }

    for(i=0;i<2*MAX_ARRAY_SIZE;i++){
    	for(j=0;j<4*MAX_ARRAY_SIZE;j++){
	    inputc = maze_array[i][j];
	    out1 = 0x00; out2 = 0x00; out3 = 0x00;
	    if(inputc&(char)NORTH){
		out1 |= (char) 0x00; out2 |= (char) 0x00; out3 |= (char) 0xFF;	
	    }
	    if(inputc&(char)WEST){
		out1 |= (char) 0x00; out2 |= (char) 0xFF; out3 |= (char) 0x00;	
	    }
	    if(inputc&(char)SOUTH){
		out1 |= (char) 0x00; out2 |= (char) 0x00; out3 |= (char) 0xFF;	
	    }
	    if(inputc&(char)EAST){
		out1 |= (char) 0x00; out2 |= (char) 0xFF; out3 |= (char) 0x00;	
	    }
	    switch (inputc){
	    case (char)WALL:
	        out1 = (char) 0x00; out2 = (char) 0x00; out3 = (char) 0x00; break;
	    case (char)NO_WALL:
	        out1 = (char) 0xFF; out2 = (char) 0xFF; out3 = (char) 0xFF; break;
	    case (char)CRUMB:
	        out1 = (char) 0x00; out2 = (char) 0x00; out3 = (char) 0xFF; break;
	    case (char)NO_CRUMB:
	        out1 = (char) 0xFF; out2 = (char) 0x00; out3 = (char) 0x00; break;
	    case (char)EXIT:
	        out1 = (char) 0x00; out2 = (char) 0xFF; out3 = (char) 0x00; break;
	    }
	    fputc(out1,fp2);
	    fputc(out2,fp2);
	    fputc(out3,fp2);

	}
    }

    fclose(fp1);
    fclose(fp2);
    return 0;
}
